@@ -8,18 +8,18 @@ our @ISA = qw(Exporter);
 
 our @EXPORT_OK = qw(cpp2xs);
 
-our $VERSION = '0.24';
-$VERSION = eval $VERSION;
+our $VERSION = '0.25';
+#$VERSION = eval $VERSION;
 
 my $config_options;
 
-our @allowable_config_keys = ('AUTOWRAP', 'AUTO_INCLUDE', 'CODE', 'DIST', 'TYPEMAPS', 'LIBS', 'INC', 
+our @allowable_config_keys = ('AUTOWRAP', 'AUTO_INCLUDE', 'CODE', 'DIST', 'TYPEMAPS', 'LIBS', 'INC',
         'WRITE_MAKEFILE_PL', 'BUILD_NOISY', 'BOOT', 'BOOT_F', 'EXPORT_ALL', 'EXPORT_OK_ALL',
         'EXPORT_TAGS_ALL', 'MAKE', 'PREFIX', 'PREREQ_PM', 'CCFLAGS', 'CCFLAGSEX', 'LD', 'LDDLFLAGS',
-        'MYEXTLIB', 'OPTIMIZE', 'PRE_HEAD', 'CC', 'SRC_LOCATION', '_TESTING', 'USE', 'USING',
+        'MYEXTLIB', 'OPTIMIZE', 'PRE_HEAD', 'CC', 'SRC_LOCATION', 'T', '_TESTING', 'USE', 'USING',
         'WRITE_PM', 'VERSION', 'MANIF');
 
-##=========================##  
+##=========================##
 
 sub cpp2xs {
     ## This is basically just a copy'n'paste of the InlineX::C2XS::c2xs() function,
@@ -31,20 +31,24 @@ sub cpp2xs {
 
     # Set the default for $build_dir.
     # (This will be overwritten by the build_dir argument in @_, if supplied.)
-    my $build_dir = '.'; 
+    my $build_dir = '.';
+
 
-    
     if(@_) {
       if(ref($_[0]) eq "HASH") {
-        $config_options = shift; 
+        $config_options = shift;
         if(@_) {die "Incorrect usage - there should be no arguments to cpp2xs() after the hash reference"}
+        # Check for invalid config options - and die if one is found
+        for(keys(%$config_options)) { die "$_ is an invalid config option" if !_check_config_keys($_)}
       }
       else {$build_dir = shift}
     }
 
     if(@_) {
-      if(ref($_[0]) ne "HASH") {die "Fourth arg to cpp2xs() needs to be a hash containing config options ... but it's not !!\n"}  
+      if(ref($_[0]) ne "HASH") {die "Fourth arg to cpp2xs() needs to be a hash containing config options ... but it's not !!\n"}
       $config_options = shift;
+      # Check for invalid config options - and die if one is found
+      for(keys(%$config_options)) { die "$_ is an invalid config option" if !_check_config_keys($_)}
     }
 
     unless(-d $build_dir) {
@@ -69,7 +73,7 @@ sub cpp2xs {
     }
     elsif(exists($config_options->{SRC_LOCATION})) {
       open(RD, "<", $config_options->{SRC_LOCATION}) or die "Can't open ", $config_options->{SRC_LOCATION}, " for reading: $!";
-      while(<RD>) { 
+      while(<RD>) {
            $code .= $_;
            if($_ =~ /inline_stack_vars/i) {$need_inline_h = 1}
       }
@@ -77,7 +81,7 @@ sub cpp2xs {
     }
     else {
       open(RD, "<", "src/$modfname.cpp") or die "Can't open src/${modfname}.cpp for reading: $!";
-      while(<RD>) { 
+      while(<RD>) {
            $code .= $_;
            if($_ =~ /inline_stack_vars/i) {$need_inline_h = 1}
       }
@@ -129,25 +133,23 @@ sub cpp2xs {
     $o->{API}{module} = $module;
     $o->{API}{code} = $code;
 
-    # Check for invalid config options - and die if one is found
-    for(keys(%$config_options)) { die "$_ is an invalid config option" if !_check_config_keys($_)}
-
     if(exists($config_options->{BUILD_NOISY})) {$o->{CONFIG}{BUILD_NOISY} = $config_options->{BUILD_NOISY}}
 
     if($config_options->{DIST}) {
       $config_options->{WRITE_MAKEFILE_PL} = 'P';
       $config_options->{WRITE_PM} = 1;
       $config_options->{MANIF} = 1;
+      $config_options->{T} = 1;
     }
 
     if($config_options->{AUTOWRAP}) {$o->{ILSM}{AUTOWRAP} = 1}
 
-    if($config_options->{BOOT}) {$o->{ILSM}{XS}{BOOT} = $config_options->{BOOT}} 
+    if($config_options->{BOOT}) {$o->{ILSM}{XS}{BOOT} = $config_options->{BOOT}}
 
     if($config_options->{BOOT_F}) {
       my $code;
       open(RD, "<", $config_options->{BOOT_F}) or die "Can't open ", $config_options->{BOOT_F}, " for reading: $!";
-      while(<RD>) { 
+      while(<RD>) {
            $code .= $_;
            if($_ =~ /inline_stack_vars/i) {$need_inline_h = 1}
       }
@@ -171,7 +173,7 @@ sub cpp2xs {
         for(@vals) {
            die "Couldn't locate the typemap $_" unless -f $_;
         }
-        $o->{ILSM}{MAKEFILE}{TYPEMAPS} = \@vals; 
+        $o->{ILSM}{MAKEFILE}{TYPEMAPS} = \@vals;
       }
     }
     else {
@@ -226,7 +228,7 @@ sub cpp2xs {
       #} # close scope
     }
 
-    if($config_options->{AUTO_INCLUDE}) {$o->{ILSM}{AUTO_INCLUDE} .= $config_options->{AUTO_INCLUDE} . "\n"} 
+    if($config_options->{AUTO_INCLUDE}) {$o->{ILSM}{AUTO_INCLUDE} .= $config_options->{AUTO_INCLUDE} . "\n"}
 
     if($config_options->{CC}) {$o->{ILSM}{MAKEFILE}{CC} = $config_options->{CC}}
 
@@ -302,6 +304,10 @@ sub cpp2xs {
     if($config_options->{MANIF}) {
       _write_manifest($modfname, $build_dir, $config_options, $need_inline_h);
     }
+
+    if($config_options->{T}) {
+      _write_test_script($module, $build_dir);
+    }
 }
 
 ##=========================##
@@ -311,7 +317,7 @@ sub _build {
     my $need_inline_headers = shift;
     my $portable = shift;
     my $save;
-    
+
     $o->call('preprocess', 'Build Preprocess');
     $o->call('parse', 'Build Parse');
 
@@ -363,7 +369,7 @@ sub _write_pm {
     my @use;
 
     if($config_options->{USE}) {
-      die "Value supplied to config option USE must be an array reference" 
+      die "Value supplied to config option USE must be an array reference"
         if ref($config_options->{USE}) ne 'ARRAY';
       @use = @{$config_options->{USE}};
     }
@@ -380,7 +386,7 @@ sub _write_pm {
     print WR "\n";
     print WR "require Exporter;\n*import = \\&Exporter::import;\nrequire DynaLoader;\n\n";
     print WR "our \$VERSION = '", $o->{API}{version}, "';\n";
-    print WR "\$VERSION = eval \$VERSION;\n"; 
+    print WR "\$VERSION = eval \$VERSION;\n";
     print WR "DynaLoader::bootstrap ", $o->{API}{module}, " \$VERSION;\n\n";
 
     unless($config_options->{EXPORT_ALL}) {
@@ -390,7 +396,7 @@ sub _write_pm {
       print WR "\@", $o->{API}{module}, "::EXPORT = qw(\n";
       for(@{$o->{ILSM}{parser}{data}{functions}}) {
         # exclude function names that begin with a single underscore
-        if ($_ =~ /^_/  && $_ !~ /^__/)  {next} 
+        if ($_ =~ /^_/  && $_ !~ /^__/)  {next}
         my $l = length($_);
         if($length + $l > $max) {
           print WR "\n", " " x $offset, "$_ ";
@@ -411,7 +417,7 @@ sub _write_pm {
       print WR "\@", $o->{API}{module}, "::EXPORT_OK = qw(\n";
       for(@{$o->{ILSM}{parser}{data}{functions}}) {
         # exclude function names that begin with a single underscore
-        if ($_ =~ /^_/ && $_ !~ /^__/)   {next} 
+        if ($_ =~ /^_/ && $_ !~ /^__/)   {next}
         my $l = length($_);
         if($length + $l > $max) {
           print WR "\n", " " x $offset, "$_ ";
@@ -612,7 +618,7 @@ sub rewrite_makefile_pl {
 >>
 >>    print WRXS "\n/* This file generated by the Makefile.PL */\n\n";
 >>    print WRXS $standard;
->>    print WRXS $auto; 
+>>    print WRXS $auto;
 >>
 >>    close WRXS or die "Couldn't close icppdefines.h: $!";
 >>
@@ -665,6 +671,25 @@ sub _write_manifest {
 
 ##=========================##
 
+sub _write_test_script {
+  use File::Path;
+  my $mod = $_[0];
+  my $path = "$_[1]/t";
+  unless(-d $path) {
+    if(!File::Path::make_path($path, {verbose => 1})) {die "Failed to create $path directory"};
+  }
+
+  print "Writing 00load.t in the $path directory\n";
+  open WRT, '>', "$path/00load.t" or die "Couldn't open $path/00load.t for writing: $!";
+  print WRT "## This file auto-generated by InlineX-CPP2XS-" . $InlineX::CPP2XS::VERSION . "\n\n";
+  print WRT "use strict;\nuse warnings;\n\n";
+  print WRT "print \"1..1\\n\";\n\n";
+  print WRT "eval{require $mod;};\n";
+  print WRT "if(\$\@) {\n  warn \"\\\$\\\@: \$\@\";\n  print \"not ok 1\\n\";\n}\n";
+  print WRT "else {print \"ok 1\\n\"}\n";
+  close WRT or die "Couldn't close $path/00load.t after writing: $!";
+}
+
 ##=========================##
 
 ##=========================##
@@ -704,7 +729,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
   # if that file is going to be needed to build the module:
   cpp2xs($module_name, $package_name, $build_dir);
 
-  # Alternatively create XS_MOD.xs (and CPP.map, if needed) in 
+  # Alternatively create XS_MOD.xs (and CPP.map, if needed) in
   # the cwd:
   cpp2xs($module_name, $package_name);
 
@@ -778,7 +803,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
  InlineX::CPP2XS looks for the source file in ./src directory - expecting
  that the filename will be the same as what appears after the final '::'
  in the module name (with a '.cpp' extension). ie if your module is
- called My::Next::Mod the cpp2xs() function looks for a file ./src/Mod.cpp, 
+ called My::Next::Mod the cpp2xs() function looks for a file ./src/Mod.cpp,
  and creates a file named Mod.xs.  Also created (by the cpp2xs function,
  is the file 'INLINE.h' - but only if that file is needed. The generated
  xs file (and INLINE.h) will be written to the cwd unless a third argument
@@ -786,7 +811,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
 
  The created XS file, when packaged with the '.pm' file, an
  appropriate 'Makefile.PL' (which can also be auto-generated by setting
- the WRITE_MAKEFILE_PL hash key), and 'INLINE.h' (if it's needed), can be 
+ the WRITE_MAKEFILE_PL hash key), and 'INLINE.h' (if it's needed), can be
  used to build the module in the usual way - without any dependence
  upon the Inline::CPP module. The cpp2xs() function may also produce
  a typemap file ('CPP.map'). That file, if produced, is also needed for a
@@ -822,7 +847,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
 
  BOOT_F
    Specifies a file containing C/CPP code to be executed in the XS BOOT
-   section. 
+   section.
    eg:
 
     BOOT_F => '/home/me/boot_code.ext',
@@ -925,7 +950,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
    The value(s) specified become the LIBS search path. It makes sense
    to assign this key only if WRITE_MAKEFILE_PL is set to a true value.
    eg:
- 
+
     LIBS => '-L/somewhere -lsomelib -L/elsewhere -lotherlib',
     LIBS => ['-L/somewhere -lsomelib', '-L/elsewhere -lotherlib'],
   ----
@@ -946,7 +971,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
   ----
 
   MYEXTLIB
-   Specifies a user compiled object that should be linked in. 
+   Specifies a user compiled object that should be linked in.
    Corresponds to the MakeMaker parameter. It makes sense to assign this
    key only when WRITE_MAKEFILE_PL is set to a true value. eg:
 
@@ -961,7 +986,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
   ----
 
   PREFIX
-   Specifies a prefix that will be automatically stripped from C 
+   Specifies a prefix that will be automatically stripped from C
    functions when they are bound to Perl. eg:
 
     PREFIX => 'FOO_',
@@ -974,11 +999,11 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
    else that might have been added to AUTO_INCLUDE by the user). If the
    specified value identifies a file, the contents of that file will be
    inserted, otherwise the specified value is inserted.
-   If the specified value is code, then since that code contains a "\n"
-   (as all code *should* terminate with at least one "\n"), you will get
-   a warning about an "Unsuccessful stat on filename containing newline"
-   when the test for the existence of a file that matches the PRE_HEAD
-   value is conducted.   
+   If the specified value is a string of code, then since that string
+   ends in "\n" (as all code *should* terminate with at least one "\n"),
+   you will get a warning about an "Unsuccessful stat on filename
+   containing newline" when the test for the existence of a file that
+   matches the PRE_HEAD value is conducted.
 
     PRE_HEAD => $code_or_filename;
   ----
@@ -1002,6 +1027,14 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
     SRC_LOCATION => '/home/me/source.ext',
   ----
 
+  T
+   Is false by default but, When set to true will write a basic
+   t/00load.t test script in the build directory.
+   eg:
+
+    T => 1,
+  ----
+
   TYPEMAPS
    The value(s) specified are added to the list of typemaps.
    eg:
@@ -1039,7 +1072,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
    Set this to to a true value if you want the Makefile.PL to be
    generated. (You should also assign the 'VERSION' key to the
    correct value when WRITE_MAKEFILE_PL is set.) eg:
-    
+
     WRITE_MAKEFILE_PL => 1,   # Makefile.PL and xs file may not work on
                               # a different machine.
     WRITE_MAKEFILE_PL => 'p', # Makefile.PL and xs file should be portable
@@ -1047,8 +1080,8 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
 
   WRITE_PM
    Set this to a true value if you want a .pm file to be generated.
-   You'll also need to assign the 'VERSION' key appropriately. 
-   Note that it's a fairly simplistic .pm file - no POD, no perl 
+   You'll also need to assign the 'VERSION' key appropriately.
+   Note that it's a fairly simplistic .pm file - no POD, no perl
    subroutines, no exported subs (unless EXPORT_ALL or EXPORT_OK_ALL
    has been set), no warnings - but it will allow the utilisation of all of
    the XSubs in the XS file. eg:
@@ -1063,7 +1096,7 @@ InlineX::CPP2XS - Convert from Inline C++ code to XS.
 
 =head1 LICENSE
 
-   This program is free software; you may redistribute it and/or 
+   This program is free software; you may redistribute it and/or
    modify it under the same terms as Perl itself.
    Copyright 2006-2009, 2010-12, Sisyphus
 
